//----------------------------------------------------------------------------------------------------------------------  
//
//  gRabbit - Graph Builder for Basic Implementation Tests
//  Copyright 2004, Nelson Guedes Paulo Junior (npaulo@linux.ime.usp.br)
//
//  Under gRabbit license (see LICENSE.TXT for details)
//
//
// - gRabbit Animation Iterator Class (implementation)
// 
//----------------------------------------------------------------------------------------------------------------------  
#ifdef __GNUG__
  #pragma implementation
#endif

//------------------------------------------------------------------------------//
// OBS: the iterator is responsible off deleting all elements put inside of it! //
//------------------------------------------------------------------------------//

// gRabbit includes
#include "grabbit_common.h"
#include "grabbit_animation_iterator.h"

// C++ includes
#include <iostream>
#include <fstream>
#include <iomanip>
#include <list>
#include <stack>


//----------------------------------------------------------------------------------------------------------------------
// Class Definition
//----------------------------------------------------------------------------------------------------------------------
GrabbitAnimationIterator::GrabbitAnimationIterator() :
  actual(NULL), cicle(false)
{
  GrabbitInstructions *cabeca = new GrabbitInstructions();
  cabeca->SetType( GrabbitInstructions::Finalization );
  instructions.push_back( cabeca );
}

//----------------------------------------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------------------------------------
GrabbitAnimationIterator::~GrabbitAnimationIterator()
{
  std::list< GrabbitInstructions * >::iterator it;
  for (it=instructions.begin();it != instructions.end();++it) {
    delete (*it);
  }
  instructions.clear();
  
  while (!undo.empty()) {
    delete undo.top();
    undo.pop();
  }
}

//----------------------------------------------------------------------------------------------------------------------
// Get a cicle
//----------------------------------------------------------------------------------------------------------------------
bool GrabbitAnimationIterator::GetCicle()
{
  bool tmp = cicle;
  if (cicle)
    cicle = false;
  return tmp;
}

//----------------------------------------------------------------------------------------------------------------------
// Inserts a new element on the queue at the current end.
//----------------------------------------------------------------------------------------------------------------------
void GrabbitAnimationIterator::InsertIntruction(GrabbitInstructions *grb)
{
  instructions.push_back( grb );
  actual = grb;
}

//----------------------------------------------------------------------------------------------------------------------
// Initialize correctly the iterator. Must be performed before the first use.
//----------------------------------------------------------------------------------------------------------------------
void GrabbitAnimationIterator::Initialize()
{
  if ((instructions.front())->GetType() == GrabbitInstructions::Finalization) {
    GrabbitInstructions *tmp = instructions.front();
    instructions.pop_front();
    instructions.push_back( tmp );
  }
}

//----------------------------------------------------------------------------------------------------------------------
// Insert a new Undo instruction.
//----------------------------------------------------------------------------------------------------------------------
void GrabbitAnimationIterator::InsertUndoIntruction(GrabbitInstructions *grb)
{
  undo.push( grb );
}

//----------------------------------------------------------------------------------------------------------------------
// Clear all instructions
//----------------------------------------------------------------------------------------------------------------------
void GrabbitAnimationIterator::ClearInstructions()
{
  std::list< GrabbitInstructions * >::iterator it;
  for (it=instructions.begin();it != instructions.end();++it) {
    delete (*it);
  }
  instructions.clear();
  
  GrabbitInstructions *cabeca = new GrabbitInstructions();
  cabeca->SetType( GrabbitInstructions::Finalization );
  instructions.push_back(cabeca);

  actual = NULL;
  
  while (!undo.empty()) {
    delete undo.top();
    undo.pop();
  }
}

//----------------------------------------------------------------------------------------------------------------------
// How many instructions does it take to screw in a lightbulb? :-)
//----------------------------------------------------------------------------------------------------------------------
unsigned long GrabbitAnimationIterator::Size()
{
  return instructions.size();
}

//----------------------------------------------------------------------------------------------------------------------
// Scroll the queue until the especial instruction is founded
//----------------------------------------------------------------------------------------------------------------------
void GrabbitAnimationIterator::Restart()
{
  GrabbitInstructions *tmp;
  while ( (tmp=instructions.front())->GetType() != GrabbitInstructions::Finalization ) {
    instructions.pop_front();
    instructions.push_back( tmp );
  }
  instructions.pop_front();
  instructions.push_back( tmp );
  actual = instructions.front();
  cicle = false;
  
  while (!undo.empty()) {
    delete undo.top();
    undo.pop();
  }
}

//----------------------------------------------------------------------------------------------------------------------
// Returns the actual instruction to the caller
//----------------------------------------------------------------------------------------------------------------------
GrabbitInstructions *GrabbitAnimationIterator::GetActualInstruction()
{
  return actual;
}

//----------------------------------------------------------------------------------------------------------------------
// Undo one instruction
//----------------------------------------------------------------------------------------------------------------------
GrabbitInstructions *GrabbitAnimationIterator::Undo()
{
  GrabbitInstructions *tmp;
  if (instructions.size()) {
    tmp = instructions.back();
    instructions.pop_back();
    instructions.push_front( tmp );
    if ( tmp->GetType() == GrabbitInstructions::Finalization ) {
      tmp = instructions.back();
      instructions.pop_back();
      instructions.push_front( tmp );
    }
    actual = tmp;
  }
  tmp = NULL;
  if (undo.size()) {
    tmp = undo.top();
    undo.pop();
  }
  return tmp;
}

//----------------------------------------------------------------------------------------------------------------------
// Returns the next instruction to the caller
//----------------------------------------------------------------------------------------------------------------------
GrabbitInstructions *GrabbitAnimationIterator::GetNextInstruction()
{
  GrabbitInstructions *tmp = NULL;
  actual = NULL;
  if (instructions.size()) {
    tmp = instructions.front();
    instructions.pop_front();
    instructions.push_back( tmp );
    if ( tmp->GetType() == GrabbitInstructions::Finalization ) {
      tmp = instructions.front();
      instructions.pop_front();
      instructions.push_back( tmp );
      cicle = true;
    }
    actual = tmp;
  }
  return tmp;
}
