//----------------------------------------------------------------------------------------------------------------------
//
//  gRabbit - Graph Builder for Basic Implementation Tests
//  Copyright 2004, Nelson Guedes Paulo Junior (npaulo@linux.ime.usp.br)
//
//  Under gRabbit license (see LICENSE.TXT for details)
//
//
//  - GrabbitBasicShape: Abstract class for building shapes (interface)
// 
//----------------------------------------------------------------------------------------------------------------------
#ifndef __GRABBIT_BASIC_SHAPE_H_
#define __GRABBIT_BASIC_SHAPE_H_

#ifdef __GNUG__
  #pragma interface
#endif

// gRabbit includes
#include "grabbit_common.h"

#define GRB_DEFAULT_PEN_SIZE 2

//----------------------------------------------------------------------------------------------------------------------
// Class Definition
//----------------------------------------------------------------------------------------------------------------------
class GrabbitBasicShape
{
 public:
  GrabbitBasicShape(const wxPoint &pos, const wxSize &tam, const wxColour &col)
    : shape_pos(pos), shape_tam(tam), shape_col(col), name(""), label(""), shape_pen_size(GRB_DEFAULT_PEN_SIZE), 
    shape_bmp(tam.x, tam.y, -1) { }
  GrabbitBasicShape() : shape_col(0, 0, 0), shape_tam(wxDefaultSize), shape_pos(wxDefaultPosition),
    shape_pen_size(GRB_DEFAULT_PEN_SIZE) { }
  virtual ~GrabbitBasicShape() { }

  // Get Methods
  const wxPoint& GetPosition() const { return shape_pos; }
  const wxColour& GetColour() const { return shape_col; }
  const wxSize& GetSize() const { return shape_tam; }
  const wxString& GetName() const { return name; }
  const wxString& GetLabel() { return label; }
  const unsigned int GetPenSize() const { return shape_pen_size; }
  
  // Set Methods
  void SetName(const wxString& n) { name = n; }
  void SetLabel(const wxString& n) { label = n; }
  void SetColour(const wxColour& col) { shape_col = col; }
  void SetSize(const wxSize& size) { shape_tam = size; }
  void SetPos(const wxPoint& pos) { shape_pos = pos; }
  void SetPenSize(const unsigned int size) { shape_pen_size = size; }
  
  // Other
  virtual wxPoint GetCentre() const {
    return wxPoint((shape_pos.x + shape_tam.x / 2) + 1, (shape_pos.y + shape_tam.y / 2) + 1 );
  }
  virtual void Move(const wxPoint& pos) { shape_pos = pos; }
  virtual void Move(long x, long y) { shape_pos.x = x;  shape_pos.y = y;  }
  virtual void Draw(wxDC& dc);
  
  // Static
  static void SetBgColour(wxColour col) { GrabbitBasicShape::bg_colour = col; }
  static const wxColour &GetBgColour() { return GrabbitBasicShape::bg_colour; }
  
  // To implement in derived class
  virtual bool GotInside(long x, long y) const = 0;
  virtual void RefreshImage() = 0;
  virtual void Highlight() = 0;
  
 protected:
  wxString name;
  wxString label;
  wxPoint shape_pos;
  wxSize shape_tam;
  wxColour shape_col;
  wxBitmap shape_bmp;
  unsigned int shape_pen_size;
  static wxColour bg_colour;
};

#endif
