//----------------------------------------------------------------------------------------------------------------------
//
//  gRabbit - Graph Builder for Basic Implementation Tests
//  Copyright 2004, Nelson Guedes Paulo Junior (npaulo@linux.ime.usp.br)
//
//  Under gRabbit license (see LICENSE.TXT for details)
//
//
// - gRabbit Animation Instructions Class (interface)
//
//----------------------------------------------------------------------------------------------------------------------
#ifndef __GRABBIT_INSTRUCTIONS_H_
#define __GRABBIT_INSTRUCTIONS_H_

#ifdef __GNUG__
  #pragma interface
#endif

// gRabbit includes
#include "grabbit_common.h"

//----------------------------------------------------------------------------------------------------------------------
// Class Definition
//----------------------------------------------------------------------------------------------------------------------
class GrabbitInstructions {
 public:
  GrabbitInstructions();
  ~GrabbitInstructions();

  // Enum
  enum InstrTypes {
    Delay=0, LoadGraph, SaveGraph, PaintVertex, PaintArc, ChangeVertexText, ChangeArcText,
    Finalization
  };
  
  // Set methods
  void SetName(const wxString &n) { name = n; }
  void SetLabel(const wxString &l) { label = l; }
  void SetColor(const unsigned char r, const unsigned char g, const unsigned char b) { shape_col = wxColour(r,g,b); }
  void SetColor(const wxColour c) { shape_col = c; }
  void SetType(const enum InstrTypes &t) { type = t; }
  void SetTime(const unsigned long t) { time = t; }
  void SetPenSize(const unsigned int s) { pen_size = (s>0)?s:2; }
  
  // Get Methods
  const unsigned long &GetTime() { return time; }
  const wxString &GetName() { return name; }
  const wxString &GetLabel() { return label; }
  const wxColour &GetColour() { return shape_col; }
  const unsigned int GetPenSize() { return pen_size; }
  enum InstrTypes GetType() { return type; }
  
  
 private:
  wxString name;
  wxString label;
  wxColour shape_col;
  unsigned long time;
  unsigned int pen_size;
  enum InstrTypes type;
};

#endif
