//----------------------------------------------------------------------------------------------------------------------  
//
//  gRabbit - Graph Builder for Basic Implementation Tests
//  Copyright 2004, Nelson Guedes Paulo Junior (npaulo@linux.ime.usp.br)
//
//  Under gRabbit license (see LICENSE.TXT for details)
//
//
// - Main Animation Panel Class (implementation)
// 
//----------------------------------------------------------------------------------------------------------------------  
#ifdef __GNUG__
  #pragma implementation
#endif

// gRabbit includes
#include "grabbit_common.h"
#include "grabbit_config.h"
#include "grabbit_main_animation_panel.h"
#include "grabbit_main_scrolled_window.h"

// External code
#include "toggle.h"

// wxWidgets Includes
#include "wx/tglbtn.h"

// XPM Files
#include "play.xpm"
#include "stop.xpm"
#include "pause.xpm"
#include "round.xpm"
#include "step_back.xpm"
#include "step_next.xpm"
#include "rewind.xpm"
#include "forward.xpm"

// C++ Library includes
#include <vector>
#include <iostream>

//----------------------------------------------------------------------------------------------------------------------
// Static
//----------------------------------------------------------------------------------------------------------------------
GrabbitMainAnimationPanel *GrabbitMainAnimationPanel::instance = NULL;

//----------------------------------------------------------------------------------------------------------------------  
// Global Magic Numbers :-)
//----------------------------------------------------------------------------------------------------------------------  
#define BUTTON_SIZE 26


//----------------------------------------------------------------------------------------------------------------------  
// WINDOW EVENT TABLE
//----------------------------------------------------------------------------------------------------------------------  
// OBS: We need the two tipe of events, otherwise some events may be missed and you get two buttons pressed at
//      the same time.
//----------------------------------------------------------------------------------------------------------------------  
BEGIN_EVENT_TABLE(GrabbitMainAnimationPanel, wxPanel)

  
  EVT_CUSTOM(wxEVT_COMMAND_BUTTON_CLICKED, Button_Animation_Play+0, GrabbitMainAnimationPanel::GenericClickButtonResponse)
  EVT_CUSTOM(wxEVT_COMMAND_BUTTON_CLICKED, Button_Animation_Play+1, GrabbitMainAnimationPanel::GenericClickButtonResponse)
  EVT_CUSTOM(wxEVT_COMMAND_BUTTON_CLICKED, Button_Animation_Play+2, GrabbitMainAnimationPanel::GenericClickButtonResponse)
  EVT_CUSTOM(wxEVT_COMMAND_BUTTON_CLICKED, Button_Animation_Play+3, GrabbitMainAnimationPanel::GenericClickButtonResponse)
  EVT_CUSTOM(wxEVT_COMMAND_BUTTON_CLICKED, Button_Animation_Play+4, GrabbitMainAnimationPanel::GenericClickButtonResponse)
  EVT_CUSTOM(wxEVT_COMMAND_BUTTON_CLICKED, Button_Animation_Play+5, GrabbitMainAnimationPanel::GenericClickButtonResponse)
  EVT_CUSTOM(wxEVT_COMMAND_BUTTON_CLICKED, Button_Animation_Play+6, GrabbitMainAnimationPanel::GenericClickButtonResponse)
  EVT_CUSTOM(wxEVT_COMMAND_BUTTON_CLICKED, Button_Animation_Play+7, GrabbitMainAnimationPanel::GenericClickButtonResponse)

  EVT_SLIDER(SLIDER_ID, GrabbitMainAnimationPanel::OnSliderUpdate)
  
  EVT_SIZE(GrabbitMainAnimationPanel::OnResize)
END_EVENT_TABLE();


//----------------------------------------------------------------------------------------------------------------------  
// Constructor
//----------------------------------------------------------------------------------------------------------------------  
GrabbitMainAnimationPanel::GrabbitMainAnimationPanel(wxWindow *main)
  : wxPanel (main, -1, wxPoint(10, 10), wxSize(BUTTON_SIZE*4, BUTTON_SIZE*5), wxSTATIC_BORDER|wxTAB_TRAVERSAL )
{
  wxCustomButton *tmpButton;

  // Index inicialization
  unsigned int btIdx = Button_Animation_Play;
  selected = -1;
  
  // Constraints
  wxLayoutConstraints *lay = new wxLayoutConstraints;
  lay->left.SameAs(main, wxLeft, 10); 
  lay->width.Absolute(BUTTON_SIZE*4);
  lay->bottom.Absolute(BUTTON_SIZE*5);
  lay->top.SameAs(main, wxTop, 10);
  this->SetConstraints(lay);

  // The Animation Buttons
  { // Button for play
    tmpButton = new wxCustomButton(this,
				   btIdx,
				   wxString(""),
				   wxBitmap(play_xpm),
				   wxDefaultPosition,
				   wxSize(BUTTON_SIZE, BUTTON_SIZE),
				   wxCUSTBUT_TOGGLE);
    tmpButton->SetToolTip(_T("Play Animation"));
    grb_buttons.push_back(tmpButton);
    tmpButton->SetValue(false);
    ++btIdx;
  }
  { // Button for stop
    tmpButton = new wxCustomButton(this,
				   btIdx,
				   wxString(""),
				   wxBitmap(stop_xpm),
				   wxDefaultPosition,
				   wxSize(BUTTON_SIZE, BUTTON_SIZE),
				   wxCUSTBUT_TOGGLE);
    tmpButton->SetToolTip(_T("Stop Animation"));
    grb_buttons.push_back(tmpButton);
    tmpButton->SetValue(false);
    ++btIdx;
  }
  { // Button for pause
    tmpButton = new wxCustomButton(this,
				   btIdx,
				   wxString(""),
				   wxBitmap(pause_xpm),
				   wxDefaultPosition,
				   wxSize(BUTTON_SIZE,BUTTON_SIZE),
				   wxCUSTBUT_TOGGLE);
    tmpButton->SetToolTip(_T("Pause Animation"));
    grb_buttons.push_back(tmpButton);
    tmpButton->SetValue(false);
    ++btIdx;
  }
  { // Button for round
    tmpButton = new wxCustomButton(this,
				   btIdx,
				   wxString(""),
				   wxBitmap(round_xpm),
				   wxDefaultPosition,
				   wxSize(BUTTON_SIZE,BUTTON_SIZE),
				   wxCUSTBUT_TOGGLE);
    tmpButton->SetToolTip(_T("Continuous Animation"));
    grb_buttons.push_back(tmpButton);
    tmpButton->SetValue(false);
    ++btIdx;
  }
  { // Button for Step Back
    tmpButton = new wxCustomButton(this,
				   btIdx,
				   wxString(""),
				   wxBitmap(step_back_xpm),
				   wxDefaultPosition,
				   wxSize(BUTTON_SIZE,BUTTON_SIZE),
				   wxCUSTBUT_TOGGLE);
    tmpButton->SetToolTip(_T("Step Back"));
    grb_buttons.push_back(tmpButton);
    tmpButton->SetValue(false);
    ++btIdx;
  }
  { // Button for Step Next
    tmpButton = new wxCustomButton(this,
				   btIdx,
				   wxString(""),
				   wxBitmap(step_next_xpm),
				   wxDefaultPosition,
				   wxSize(BUTTON_SIZE,BUTTON_SIZE),
				   wxCUSTBUT_BUTTON);
    tmpButton->SetToolTip(_T("Step Next"));
    grb_buttons.push_back(tmpButton);
    tmpButton->SetValue(false);
    ++btIdx;
  }
  { // Button for Rewind
    tmpButton = new wxCustomButton(this,
				   btIdx,
				   wxString(""),
				   wxBitmap(rewind_xpm),
				   wxDefaultPosition,
				   wxSize(BUTTON_SIZE,BUTTON_SIZE),
				   wxCUSTBUT_BUTTON);
    tmpButton->SetToolTip(_T("Rewind Animation"));
    grb_buttons.push_back(tmpButton);
    tmpButton->SetValue(false);
    ++btIdx;
  }
  { // Button for Forward
    tmpButton = new wxCustomButton(this,
				   btIdx,
				   wxString(""),
				   wxBitmap(forward_xpm),
				   wxDefaultPosition,
				   wxSize(BUTTON_SIZE,BUTTON_SIZE),
				   wxCUSTBUT_BUTTON);
    tmpButton->SetToolTip(_T("Forward Animation"));
    grb_buttons.push_back(tmpButton);
    tmpButton->SetValue(false);
    ++btIdx;
  }

  speedBar = new wxSlider(this, SLIDER_ID, 500, 1, 5000, wxDefaultPosition, wxDefaultSize, wxSL_AUTOTICKS|wxSL_LABELS);
  speedBar->SetToolTip(_T("Animation Speed (in ms)"));
  
  
  { // Set grid and add buttons...
    wxSizer *vsizer = new wxStaticBoxSizer(new wxStaticBox(this, -1, _T("Animation"), wxDefaultPosition, wxDefaultSize, wxALIGN_CENTER),
					   wxVERTICAL);
    
    
    wxGridSizer * gridsizer = new wxGridSizer(4);
    for (unsigned int i=0;i<grb_buttons.size();++i) {
      gridsizer->Add(grb_buttons[i], 0, wxEXPAND);
    }
    vsizer->Add(gridsizer, 0, wxEXPAND);
    vsizer->Add(speedBar, 0, wxEXPAND|wxTOP, 10);
    this->SetSizer(vsizer);
  }
  
  // This... :-)
  instance = this;
  this->SetAutoLayout(true);
}

//----------------------------------------------------------------------------------------------------------------------  
// Destructor
//----------------------------------------------------------------------------------------------------------------------  
GrabbitMainAnimationPanel::~GrabbitMainAnimationPanel()
{ // Nothing to do....
}

//----------------------------------------------------------------------------------------------------------------------
// Get the button objet, or at least, a pointer..... :-)
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainAnimationPanel::DepressButtonById(unsigned int num, bool flag)
{
  int calc = num - Button_Animation_Play;
  if (calc >= grb_buttons.size() || calc < 0)
    return;
  if (!flag && (calc == selected)) selected = -1;
  if (flag) selected = calc;
  grb_buttons[calc]->SetValue( flag );
}

//----------------------------------------------------------------------------------------------------------------------
// This function updates the time value for the animation
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainAnimationPanel::OnSliderUpdate(wxCommandEvent &event)
{
  GrabbitMainScrolledWindow::SetDelay( speedBar->GetValue() );
}

//----------------------------------------------------------------------------------------------------------------------  
// This function is responsible to process the Button events.
//----------------------------------------------------------------------------------------------------------------------  
void GrabbitMainAnimationPanel::GenericClickButtonResponse(wxCommandEvent& event)
{
  // This is the most problematic function and certanly the main source for bugs in all this code... :-)
  
  static int IDX_ROUND = Button_Animation_Round - Button_Animation_Play;
  static int IDX_STOP = Button_Animation_Stop - Button_Animation_Play;
  static int IDX_STEP_BACK = Button_Animation_Step_Back - Button_Animation_Play;
  
  int theSel = ((wxWindow *)event.GetEventObject())->GetId() - Button_Animation_Play;

  if (selected != -1) {
    if (grb_buttons[selected] != event.GetEventObject()) {
      if ( theSel != IDX_ROUND && theSel != IDX_STEP_BACK) {
	grb_buttons[selected]->SetValue(false);
	selected = theSel;
      }
      if (event.GetExtraLong() != wxEVT_LEFT_UP) {
	(GrabbitMainScrolledWindow::GetMe())->ExecAnimationHandlerFunction( theSel );
      }
    } else {
      if (event.GetExtraLong() != wxEVT_LEFT_UP) {
	grb_buttons[selected]->SetValue(true);
      }
    }
  } else {
    if (event.GetExtraLong() != wxEVT_LEFT_UP) {
      if ( theSel != IDX_ROUND ) {
	selected = theSel;
      }
      (GrabbitMainScrolledWindow::GetMe())->ExecAnimationHandlerFunction( theSel );
    }
  }
  if ( theSel == IDX_STOP || theSel == IDX_STEP_BACK) {
    grb_buttons[theSel]->SetValue(false);
  }
}

//----------------------------------------------------------------------------------------------------------------------  
// OnResize
//----------------------------------------------------------------------------------------------------------------------  
void GrabbitMainAnimationPanel::OnResize(wxSizeEvent &event)
{
  this->Refresh(true);
  event.Skip();
}
