//----------------------------------------------------------------------------------------------------------------------  
//
//  gRabbit - Graph Builder for Basic Implementation Tests
//  Copyright 2004, Nelson Guedes Paulo Junior (npaulo@linux.ime.usp.br)
//
//  Under gRabbit license (see LICENSE.TXT for details)
//
//
// - Main Button Panel class (implementation)
// 
//----------------------------------------------------------------------------------------------------------------------  
#ifdef __GNUG__
  #pragma implementation
#endif

// gRabbit includes
#include "grabbit_common.h"
#include "grabbit_config.h"
#include "grabbit_main_button_panel.h"
#include "grabbit_main_scrolled_window.h"

// External code
#include "toggle.h"

// wxWidgets Includes
#include "wx/tglbtn.h"

// XPM Files
#include "bola2.xpm"
#include "edge.xpm"
#include "hand.xpm"

// C++ Library includes
#include <vector>
#include <iostream>

//----------------------------------------------------------------------------------------------------------------------  
// Global Magic Numbers :-)
//----------------------------------------------------------------------------------------------------------------------  
#define BUTTON_SIZE 26


//----------------------------------------------------------------------------------------------------------------------  
// WINDOW EVENT TABLE
//----------------------------------------------------------------------------------------------------------------------  
// OBS: We need the two tipe of events, otherwise some events may be missed and you get two buttons pressed at
//      the same time.
//----------------------------------------------------------------------------------------------------------------------  
BEGIN_EVENT_TABLE(GrabbitMainButtonPanel, wxPanel)
  EVT_CUSTOM(wxEVT_COMMAND_TOGGLEBUTTON_CLICKED, Button_Node+0, GrabbitMainButtonPanel::GenericClickButtonResponse)
  EVT_CUSTOM(wxEVT_COMMAND_TOGGLEBUTTON_CLICKED, Button_Node+1, GrabbitMainButtonPanel::GenericClickButtonResponse)
  EVT_CUSTOM(wxEVT_COMMAND_TOGGLEBUTTON_CLICKED, Button_Node+2, GrabbitMainButtonPanel::GenericClickButtonResponse)
  
  EVT_CUSTOM(wxEVT_COMMAND_BUTTON_CLICKED, Button_Node+0, GrabbitMainButtonPanel::GenericClickButtonResponse)
  EVT_CUSTOM(wxEVT_COMMAND_BUTTON_CLICKED, Button_Node+1, GrabbitMainButtonPanel::GenericClickButtonResponse)
  EVT_CUSTOM(wxEVT_COMMAND_BUTTON_CLICKED, Button_Node+2, GrabbitMainButtonPanel::GenericClickButtonResponse)
  
  EVT_SIZE(GrabbitMainButtonPanel::OnResize)
END_EVENT_TABLE();


//----------------------------------------------------------------------------------------------------------------------  
// Constructor
//----------------------------------------------------------------------------------------------------------------------  
GrabbitMainButtonPanel::GrabbitMainButtonPanel(wxWindow *main)
  : wxPanel (main, -1, wxDefaultPosition, wxSize(BUTTON_SIZE*4, BUTTON_SIZE*5), wxSTATIC_BORDER|wxTAB_TRAVERSAL )
{
  wxCustomButton *tmpButton;

  // Index inicialization
  unsigned int btIdx = Button_Node;
  selected = 0;
  
  // Constraints
  wxLayoutConstraints *lay = new wxLayoutConstraints;
  lay->left.SameAs(main, wxLeft, 10); 
  lay->width.Absolute(BUTTON_SIZE*4);
  lay->bottom.Absolute(BUTTON_SIZE*5);
  lay->top.SameAs(main, wxTop, 10);
  this->SetConstraints(lay);

  // The Tool Buttons
  { // Button for the selection tool
    tmpButton = new wxCustomButton(this,
				   btIdx,
				   wxString(""),
				   wxBitmap(hand_xpm),
				   wxDefaultPosition,
				   wxSize(BUTTON_SIZE,BUTTON_SIZE),
				   wxCUSTBUT_TOGGLE);
    tmpButton->SetToolTip(_T("Selection Tool"));
    grb_buttons.push_back(tmpButton);
    tmpButton->SetValue(true);
    ++btIdx;
  }
  { // Button for a vertex
    tmpButton = new wxCustomButton(this,
				   btIdx,
				   wxString(""),
				   wxBitmap(bola2_xpm),
				   wxDefaultPosition,
				   wxSize(BUTTON_SIZE,BUTTON_SIZE),
				   wxCUSTBUT_TOGGLE);
    tmpButton->SetToolTip(_T("Vertex"));
    grb_buttons.push_back(tmpButton);
    tmpButton->SetValue(false);
    ++btIdx;
  }
  { // Button for an edge
    tmpButton = new wxCustomButton(this,
				   btIdx,
				   wxString(""),
				   wxBitmap(edge_xpm),
				   wxDefaultPosition,
				   wxSize(BUTTON_SIZE,BUTTON_SIZE),
				   wxCUSTBUT_TOGGLE);
    tmpButton->SetToolTip(_T("Arc"));
    grb_buttons.push_back(tmpButton);
    tmpButton->SetValue(false);
    ++btIdx;
  }
  
  { // Set grid and add buttons...
    wxSizer *vsizer = new wxStaticBoxSizer(new wxStaticBox(this, -1, _T("Toolbar"), wxDefaultPosition, wxDefaultSize, wxALIGN_CENTER),
					   wxVERTICAL);
    
    wxGridSizer * gridsizer = new wxGridSizer(4);
    for (unsigned int i=0;i<grb_buttons.size();++i) {
      gridsizer->Add(grb_buttons[i], 0, wxEXPAND);
    }
    vsizer->Add(gridsizer, 0, wxEXPAND);
    this->SetSizer(vsizer);
  }
  
  this->SetAutoLayout(true);
}

//----------------------------------------------------------------------------------------------------------------------  
// Destructor
//----------------------------------------------------------------------------------------------------------------------  
GrabbitMainButtonPanel::~GrabbitMainButtonPanel()
{ // Nothing to do....
}

//----------------------------------------------------------------------------------------------------------------------  
// This function is responsible to process the Button events.
//----------------------------------------------------------------------------------------------------------------------  
void GrabbitMainButtonPanel::GenericClickButtonResponse(wxCommandEvent& event)
{
  if (grb_buttons[selected] != event.GetEventObject()) {
    grb_buttons[selected]->SetValue(false);
    selected = ((wxWindow *)event.GetEventObject())->GetId() - Button_Node;
    GrabbitMainScrolledWindow::SetEventHandlerFunction(selected);
  } else {
    grb_buttons[selected]->SetValue(true);
  }
}

//----------------------------------------------------------------------------------------------------------------------  
// OnResize
//----------------------------------------------------------------------------------------------------------------------  
void GrabbitMainButtonPanel::OnResize(wxSizeEvent &event)
{
  this->Refresh(true);
  event.Skip();
}
