//----------------------------------------------------------------------------------------------------------------------
//
//  gRabbit - Graph Builder for Basic Implementation Tests
//  Copyright 2004, Nelson Guedes Paulo Junior (npaulo@linux.ime.usp.br)
//
//  Under gRabbit license (see LICENSE.TXT for details)
//
//
// - Main Frame class implementation
//
//----------------------------------------------------------------------------------------------------------------------
#ifdef __GNUG__
  #pragma implementation
#endif

// gRabbit includes
#include "grabbit_common.h"
#include "grabbit_config.h"
#include "grabbit_main_frame.h"
#include "grabbit_main_scrolled_window.h"
#include "grabbit_configurator.h"
#include "grabbit_main_animation_panel.h"


// XPM Files
#include "bola2.xpm"
#include "bola.xpm"
#include "edge.xpm"
#include "arrow.xpm"
#include "hand.xpm"

// C++ Library includes
#include <list>
#include <cstdlib>
#include <sstream>
#include <iostream>
#include <vector>


//----------------------------------------------------------------------------------------------------------------------
// Global Variables
//----------------------------------------------------------------------------------------------------------------------
GrabbitConfigurator *GrabbitMainFrame::myConfig = new GrabbitConfigurator();

//----------------------------------------------------------------------------------------------------------------------
// WINDOW EVENT TABLE
//----------------------------------------------------------------------------------------------------------------------
BEGIN_EVENT_TABLE(GrabbitMainFrame, wxFrame)
  EVT_MENU(Menu_File_Quit,  GrabbitMainFrame::OnQuit)
  EVT_MENU(Menu_File_Load,  GrabbitMainFrame::OnLoadProject)
  EVT_MENU(Menu_File_Save,  GrabbitMainFrame::OnSaveProject)

  EVT_MENU(Menu_File_Load_Animation,  GrabbitMainFrame::OnLoadAnimation)
 
  EVT_SIZE(GrabbitMainFrame::OnResize)
END_EVENT_TABLE();


//----------------------------------------------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------------------------------------------
GrabbitMainFrame::GrabbitMainFrame(wxFrame *frame, wxChar *title, int x, int y, int w, int h)
  : wxFrame(frame, -1, title, wxPoint(x, y), wxSize(w, h), wxDEFAULT_FRAME_STYLE | wxNO_FULL_REPAINT_ON_RESIZE),
    button_panel(NULL), scroll_window(NULL)
{
  // Global Configurations
  {
    GrabbitConfigurator *tmpconf = new GrabbitConfigurator();
    wxColour tmpcolour(255,255,255); // Default white
    
    tmpconf->SetDrawAreaBgColor( tmpcolour );
    GrabbitBasicShape::SetBgColour( tmpcolour );
    GrabbitMainFrame::SetConfigurator( tmpconf );
  }

  // Menu and status bar...
  CreateStatusBar();
  {
    wxMenu *menuFile = new wxMenu();
    menuFile->Append(Menu_File_Load, _T("&Load\tCtrl-L"));
    menuFile->Append(Menu_File_Save, _T("&Save\tCtrl-S"));
    menuFile->AppendSeparator();
    menuFile->Append(Menu_File_Quit, _T("E&xit\tCtrl-Q"));
    
    wxMenu *menuAnimation = new wxMenu();
    menuAnimation->Append(Menu_File_Load_Animation, _T("Load &Animation File \tCtrl-A"));
    
    
    wxMenuBar *menubar = new wxMenuBar();
    {
      menubar->Append(menuFile, _T("&File"));
      menubar->Append(menuAnimation, _T("&Animation"));
    }
    SetMenuBar(menubar);
    SetStatusText(_T("Press Ctrl-Q to exit Grabbit"));
  }

  wxWindow *mainwin = this;
  mainwin->SetAutoLayout(true);
  {
    mainwin->SetClientSize(800, 600);
    wxSize s = GetAdjustedBestSize();
    mainwin->SetSizeHints(s.GetWidth()/2, s.GetHeight()/2);
    mainwin->Refresh(false);
  }
  {
    int w,h;
    mainwin->GetClientSize(&w,&h);
    button_panel = new GrabbitMainButtonPanel( this );
    animation_panel = new GrabbitMainAnimationPanel( this );
    scroll_window = new GrabbitMainScrolledWindow( this, wxPoint(26*4+20,0), wxSize(w-(26*4+20), h) );
    {
      wxGridSizer *gridsizer = new wxGridSizer(1);
      gridsizer->Add(button_panel, 0, wxADJUST_MINSIZE|wxTOP|wxLEFT, 5);
      gridsizer->Add(animation_panel, 0, wxADJUST_MINSIZE|wxLEFT, 5);
      this->SetSizer(gridsizer);
    }
    button_panel->Refresh(false);
    animation_panel->Refresh(false);
    mainwin->Refresh(false);
  }
  mainwin->Refresh(false);
}

//----------------------------------------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------------------------------------
GrabbitMainFrame::~GrabbitMainFrame() {
}

//----------------------------------------------------------------------------------------------------------------------
// OnQuit: Called from menu File->Quit
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainFrame::OnQuit(wxCommandEvent& WXUNUSED(event)) {
  Close(TRUE);
}

//----------------------------------------------------------------------------------------------------------------------
// OnLoadProject: Called from menu File->Load
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainFrame::OnLoadProject(wxCommandEvent& event) {
  wxString filename;
  wxFileDialog dialog(this,
		      _T("Choose a file"),
		      _T(""),
		      _T(""),
		      _T("gRabbit files (*.grb)|*.grb|SGB files (*.gb)|*.gb|All files (*.*)|*.*"),
		      wxOPEN | wxHIDE_READONLY | wxCHANGE_DIR);
  
  if (dialog.ShowModal() == wxID_OK) {
    filename = dialog.GetFilename();
    scroll_window->LoadGraph( filename );
  }
}

//----------------------------------------------------------------------------------------------------------------------
// OnSaveProject: Called from menu File->Save
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainFrame::OnSaveProject(wxCommandEvent& event) {
  wxString filename;
  wxFileDialog dialog(this,
		      _T("Save file to"),
		      _T(""),
		      _T(""),
		      _T("gRabbit files (*.grb)|*.grb|SGB files (*.gb)|*.gb|All files (*.*)|*.*"),
		      wxSAVE | wxOVERWRITE_PROMPT | wxHIDE_READONLY | wxCHANGE_DIR);
  
  if (dialog.ShowModal() == wxID_OK) {
    filename = dialog.GetFilename();
    scroll_window->SaveGraph( filename );
  }
}

//----------------------------------------------------------------------------------------------------------------------
// OnLoadAnimation: Called from menu Animation->Load
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainFrame::OnLoadAnimation(wxCommandEvent& event) {
  wxString filename;
  wxFileDialog dialog(this,
		      _T("Choose an animation file"),
		      _T(""),
		      _T(""),
		      _T("gRabbit Animation File (*.gaf)|*.gaf|All files (*.*)|*.*"),
		      wxOPEN | wxHIDE_READONLY | wxCHANGE_DIR);

  if (dialog.ShowModal() == wxID_OK) {
    filename = dialog.GetFilename();
    scroll_window->LoadAnimation( filename );
  }
}

//----------------------------------------------------------------------------------------------------------------------
// Response to EVT_SIZE
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainFrame::OnResize(wxSizeEvent &event) {
  if (scroll_window)
    scroll_window->OnResize(event);
  if (button_panel)
    button_panel->OnResize(event);
  event.Skip();
}
