//----------------------------------------------------------------------------------------------------------------------
//
//  gRabbit - Graph Builder for Basic Implementation Tests
//  Copyright 2004, Nelson Guedes Paulo Junior (npaulo@linux.ime.usp.br)
//
//  Under gRabbit license (see LICENSE.TXT for details)
//
//
//  - GrabbitMainScrolledWindow: Part of main window (implementation)
// 
//----------------------------------------------------------------------------------------------------------------------
#ifdef __GNUG__
  #pragma implementation
#endif

// gRabbit Includes
#include "grabbit_common.h"
#include "grabbit_config.h"
#include "grabbit_holder.h"
#include "grabbit_configurator.h"

#include "grabbit_main_scrolled_window.h"
#include "grabbit_main_animation_panel.h"
#include "grabbit_main_frame.h"

#include "grabbit_basic_shape.h"
#include "grabbit_shape_vertex.h"
#include "grabbit_shape_arc.h"
#include "grabbit_shape_line.h"

#include "grabbit_animation_parser.h"
#include "grabbit_animation_iterator.h"
#include "grabbit_instructions.h"

// wx Includes

// C++ includes
#include <iostream>
#include <fstream>
#include <iomanip>
#include <cstring>
#include <list>


// Globals
unsigned int GrabbitMainScrolledWindow::selected=0;
unsigned int GrabbitMainScrolledWindow::state=0;
unsigned long GrabbitMainScrolledWindow::delay=500;
GrabbitMainScrolledWindow *GrabbitMainScrolledWindow::instance = NULL;
wxMutex *GrabbitMainScrolledWindow::aniMutex = NULL;

//----------------------------------------------------------------------------------------------------------------------
// Event Table
//----------------------------------------------------------------------------------------------------------------------
BEGIN_EVENT_TABLE(GrabbitMainScrolledWindow, wxScrolledWindow)
  EVT_PAINT(GrabbitMainScrolledWindow::OnPaint)
  EVT_SIZE(GrabbitMainScrolledWindow::OnResize)

  EVT_LEFT_DOWN(GrabbitMainScrolledWindow::OnEvtLeftDown)

  EVT_LEFT_UP(GrabbitMainScrolledWindow::OnEvtLeftUp)
  
  EVT_MOTION(GrabbitMainScrolledWindow::OnEvtMouseMove)

  EVT_TIMER(TIMER_ID, GrabbitMainScrolledWindow::OnTimer)
END_EVENT_TABLE();

//----------------------------------------------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------------------------------------------
GrabbitMainScrolledWindow::GrabbitMainScrolledWindow(wxWindow *parent, const wxPoint& pos, const wxSize& size ):
  wxScrolledWindow(parent, -1, pos, size, wxNO_FULL_REPAINT_ON_RESIZE | wxHSCROLL | wxVSCROLL | wxRETAINED | wxSUNKEN_BORDER),
  myParent(parent), theLine(NULL), holder(NULL), myBmp(NULL), myBuffer(NULL), blankCursor(NULL), directed(true),
  myTimer(this, TIMER_ID), btnPlay(false), btnRound(false), btnPause(false), aniRestarts(false), myAniIt(NULL), actualGraph("")
{

  // Configurations
  GrabbitConfigurator *g = GrabbitMainFrame::GetConfigurator();
  SetBackgroundColour( g->GetDrawAreaBgColor() );
 
  { // Inicialize the blank cursor
    wxMemoryDC mdc;
    wxBitmap bmptmp(16, 16, 1);
    wxBitmap bitmask(16, 16, 1);
    {
      mdc.SelectObject( bmptmp );
      mdc.SetBackground(*wxBLACK_BRUSH);
      mdc.Clear();
      mdc.SelectObject( wxNullBitmap );
    }
    {
      mdc.SelectObject( bitmask );
      mdc.SetBackground(*wxWHITE_BRUSH);
      mdc.Clear();
      mdc.SelectObject( wxNullBitmap );
    }
    bmptmp.SetMask(new wxMask(bitmask));
    blankCursor = new wxCursor(wxImage(bmptmp.ConvertToImage()));
  }

    // Drawing Area
  {
    int w, h;
    this->GetClientSize(&w, &h);
    bmpW = w;
    bmpH = h;
    myBmp = new wxBitmap( bmpW, bmpH, -1);
  }


  { // Initialize Draw Area....
    wxMemoryDC mdc;
    mdc.SelectObject( *(this->myBmp) );
    mdc.SetBackground( wxBrush( g->GetDrawAreaBgColor(), wxSOLID) );
    mdc.Clear();
    mdc.SelectObject( wxNullBitmap );
  }
  this->SetScrollbars(1, 1, size.GetWidth() , size.GetHeight() );

  // Events
  events.push_back(&GrabbitMainScrolledWindow::BtnSelected);
  events.push_back(&GrabbitMainScrolledWindow::BtnNode);
  events.push_back(&GrabbitMainScrolledWindow::BtnLine);

  // Animation Handlers
  ani_events.push_back(&GrabbitMainScrolledWindow::AniButtonPlay);
  ani_events.push_back(&GrabbitMainScrolledWindow::AniButtonStop);
  ani_events.push_back(&GrabbitMainScrolledWindow::AniButtonPause);
  ani_events.push_back(&GrabbitMainScrolledWindow::AniButtonRound);
  ani_events.push_back(&GrabbitMainScrolledWindow::AniButtonStepBack);
  ani_events.push_back(&GrabbitMainScrolledWindow::AniButtonStepNext);
  ani_events.push_back(&GrabbitMainScrolledWindow::AniButtonRewind);
  ani_events.push_back(&GrabbitMainScrolledWindow::AniButtonForward);
  
  aniMutex = new wxMutex();
  
  // This... :-)
  instance = this;
  this->Refresh(true);
}

//----------------------------------------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------------------------------------
GrabbitMainScrolledWindow::~GrabbitMainScrolledWindow()
{
  if (myBmp != NULL) delete myBmp;
  if (myBuffer != NULL) delete myBuffer;
  if (blankCursor != NULL) delete blankCursor;
}


// *********************************************************************************************************************
// Event Handlers
// *********************************************************************************************************************

//----------------------------------------------------------------------------------------------------------------------
// Handles timer events for animation purposes
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::OnTimer(wxTimerEvent& event)
{
  static int timeCounter=500;
  
  aniMutex->Lock();
  if (btnPlay && (timeCounter != delay)) {
    timeCounter = delay;
    myTimer.Start( timeCounter, wxTIMER_CONTINUOUS);
    event.Skip();
    aniMutex->Unlock();
    return;
  }
  if (btnPlay) {
    this->StopTimer();
    if (!(this->ProcessNextInstruction(NULL))) { // Ops, animation has finished! Starts again?
      if (btnRound) {
	// FIX-ME: Need to select the play button
	this->myAniIt->Restart();
	// UGGHHH!!!!
	this->LoadGraph( actualGraph );
	// FIX-ME: Need to restart the Graph without reloading it from disk!
	btnPlay = true;
	this->StartTimer();
	event.Skip();
	aniMutex->Unlock();
	return;
      }
      aniRestarts = true;
      btnPlay = false;
      // FIX-ME: Need to deselect the play button
      (GrabbitMainAnimationPanel::GetMe())->DepressButtonById(Button_Animation_Play, false);
      event.Skip();
      aniMutex->Unlock();
      return;
    }
    this->StartTimer();
  }
  aniMutex->Unlock();
}

//----------------------------------------------------------------------------------------------------------------------
// Executs one animation function associated with an animation button
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::ExecAnimationHandlerFunction(unsigned int i)
{
  (this->*(ani_events[i]))();
}

//----------------------------------------------------------------------------------------------------------------------
// Process next animation instruction
//----------------------------------------------------------------------------------------------------------------------
bool GrabbitMainScrolledWindow::ProcessNextInstruction(GrabbitInstructions *undo)
{
  GrabbitInstructions *grb;
  if (undo == NULL) 
    grb = myAniIt->GetNextInstruction();
  else
    grb = undo;
  std::list< GrabbitBasicShape * >::iterator it;
  if (myAniIt->GetCicle())
    return false;
  switch (grb->GetType())
    {
    case (GrabbitInstructions::Delay):
      break;
    case (GrabbitInstructions::SaveGraph):
      break;
    case GrabbitInstructions::PaintVertex:
      for (it=all_shapes.begin();it != all_shapes.end();++it) {
	if ((*it)->GetName() == grb->GetName()) {
	  GrabbitInstructions *tmpgrb = new GrabbitInstructions();
	  tmpgrb->SetType( GrabbitInstructions::PaintVertex );
	  tmpgrb->SetName( grb->GetName() );
	  tmpgrb->SetColor( (*it)->GetColour() );
	  myAniIt->InsertUndoIntruction( tmpgrb );
	  (*it)->SetColour( grb->GetColour() );
	  (*it)->RefreshImage();
	  this->Refresh(false);
	  break;
	}
      }
      break;
    case GrabbitInstructions::PaintArc:
      for (it=all_lines.begin();it != all_lines.end();++it) {
	if ((*it)->GetName() == grb->GetName()) {
	  GrabbitInstructions *tmpgrb = new GrabbitInstructions();
	  tmpgrb->SetType( GrabbitInstructions::PaintArc );
	  tmpgrb->SetName( grb->GetName() );
	  tmpgrb->SetColor( (*it)->GetColour() );
	  myAniIt->InsertUndoIntruction( tmpgrb );
	  
	  (*it)->SetPenSize( grb->GetPenSize() );
	  (*it)->SetColour( grb->GetColour() );
	  this->Refresh(false);
	  break;
	}
      }
      break;
    case GrabbitInstructions::ChangeVertexText:
      for (it=all_shapes.begin();it != all_shapes.end();++it) {
	if ((*it)->GetName() == grb->GetName()) {
	  GrabbitInstructions *tmpgrb = new GrabbitInstructions();
	  tmpgrb->SetType( GrabbitInstructions::ChangeVertexText );
	  tmpgrb->SetName( grb->GetName() );
	  tmpgrb->SetLabel( (*it)->GetLabel() );
	  myAniIt->InsertUndoIntruction( tmpgrb );
	  (*it)->SetLabel( grb->GetLabel() );
	  (*it)->RefreshImage();
	  this->Refresh(false);
	  break;
	}
      }
      break;
    case GrabbitInstructions::ChangeArcText:
      for (it=all_lines.begin();it != all_lines.end();++it) {
	if ((*it)->GetName() == grb->GetName()) {
	  GrabbitInstructions *tmpgrb = new GrabbitInstructions();
	  tmpgrb->SetType( GrabbitInstructions::ChangeArcText );
	  tmpgrb->SetName( grb->GetName() );
	  tmpgrb->SetLabel( (*it)->GetLabel() );
	  myAniIt->InsertUndoIntruction( tmpgrb );
	  (*it)->SetLabel( grb->GetLabel() );
	  this->Refresh(false);
	  break;
	}
      }
      break;
    case GrabbitInstructions::Finalization:
      break;
    }
  return true;
}

//----------------------------------------------------------------------------------------------------------------------
// Loads the animation file
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::LoadAnimation(const wxString &filename)
{
  GrabbitAnimationParser *parser = new GrabbitAnimationParser( filename );
  myAniIt = new GrabbitAnimationIterator();
  wxString message("");
  int err;
  GrabbitInstructions *instr = NULL;
  
  while (!parser->CheckEOF()) {
    parser->ReadInstruction( myAniIt );
    if ((err=parser->CheckError()) != 0) {
      if (err != 2) { // End of File
	parser->GetError( message );
	delete myAniIt;
	delete parser;
	wxMessageBox(message, "gRabbit", wxOK | wxICON_ERROR, this);
	return;
      }
    }
    if ((myAniIt->GetActualInstruction())->GetType() == GrabbitInstructions::LoadGraph) {
      instr = myAniIt->GetActualInstruction();
    }
  }
  if (instr == NULL) {
    delete myAniIt;
    delete parser;
    message.Printf("Couldn't find a 'LoadGraph(filename)' instruction on file %s", filename.c_str());
    wxMessageBox(message, "gRabbit", wxOK | wxICON_ERROR, this);
    return;
  }
  this->LoadGraph( instr->GetName() );
  delete parser;
  aniRestarts = true;
  btnPause = false;
  btnPlay = false;
  btnRound = false;
  (GrabbitMainAnimationPanel::GetMe())->DepressButtonById(Button_Animation_Play, false);
  (GrabbitMainAnimationPanel::GetMe())->DepressButtonById(Button_Animation_Pause, false);
  (GrabbitMainAnimationPanel::GetMe())->DepressButtonById(Button_Animation_Round, false);
  this->myAniIt->Initialize();
  ((wxFrame *)myParent)->SetStatusText(_T("Animation successfully loaded...."));
}

//----------------------------------------------------------------------------------------------------------------------
// Executes the play button
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::AniButtonPlay()
{
  aniMutex->Lock();
  btnPlay = true;
  this->StartTimer();
  if (!btnPause && aniRestarts) {
    this->myAniIt->Restart();
    // UGGHHH!!!!
    this->LoadGraph( actualGraph );
    // FIX-ME: Need to restart the Graph without reloading it from disk!
    aniRestarts = false;
  }
  btnPause = false;
  aniMutex->Unlock();
}
  
//----------------------------------------------------------------------------------------------------------------------
// Stop the animation and restarts it
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::AniButtonStop()
{
  aniMutex->Lock();
  this->StopTimer();
  btnPlay = false;
  btnPause = false;
  aniRestarts = false;
  this->myAniIt->Restart();
  this->LoadGraph( actualGraph );
  aniMutex->Unlock();
}

//----------------------------------------------------------------------------------------------------------------------
// Pause the animation
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::AniButtonPause()
{
  aniMutex->Lock();
  btnPause = true;
  btnPlay = false;
  this->StopTimer();
  aniMutex->Unlock();
}

//----------------------------------------------------------------------------------------------------------------------
// Make animation continuous
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::AniButtonRound()
{
  aniMutex->Lock();
  btnRound = !btnRound;
  aniMutex->Unlock();
}

//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::AniButtonStepBack()
{
  GrabbitInstructions *tmpStr;
  
  //aniMutex->Lock();
  btnPause = true;
  btnPlay = false;
  this->StopTimer();
  if ((tmpStr=this->myAniIt->Undo()) != NULL) {
    (void)ProcessNextInstruction(tmpStr);
  }
  //aniMutex->Unlock();
}

//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::AniButtonStepNext()
{
  aniMutex->Lock();
  aniMutex->Unlock();
}

//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::AniButtonRewind()
{
  aniMutex->Lock();
  aniMutex->Unlock();
}

//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::AniButtonForward()
{
  aniMutex->Lock();
  aniMutex->Unlock();
}

//----------------------------------------------------------------------------------------------------------------------
// Starts the timer
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::StartTimer()
{
  myTimer.Start( delay, wxTIMER_CONTINUOUS );
}

//----------------------------------------------------------------------------------------------------------------------
// Stop the timer
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::StopTimer()
{
  myTimer.Stop();
}

//----------------------------------------------------------------------------------------------------------------------
// Load a previous saved graph in memory and display it
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::LoadGraph(const wxString &filename)
{
  wxString message("");
  std::list< GrabbitBasicShape * >::iterator it;
  
  { // Clear initial graph
    for (it=all_shapes.begin();it != all_shapes.end();++it) {
      delete *it;
    }
    for (it=all_lines.begin();it != all_lines.end();++it) {
      delete *it;
    }
    all_shapes.clear();
    all_lines.clear();
    this->Refresh(false);
    ((wxFrame *)myParent)->SetStatusText(_T("Loading Graph. Please wait...."));
  }
  

  std::ifstream file(filename, std::ios::in);
  if ( !file ) {
    message.Printf("Coldn't open file %s", filename.c_str());   //: %s", filename.c_str(), filename.c_str());
    wxMessageBox(message, "gRabbit", wxOK | wxICON_ERROR, this);  
    return;
  }
  file.flags ( std::ios::skipws );

  char c;
  unsigned long arcs,vertex,graph,line=1;
  { // Read Header! :-)
    char tmp[32];
    file.get( tmp, 9 );
    if ( strcmp( tmp, "GRABBIT-" ) != 0) {
      message.Printf("Invalid header file format!");
      wxMessageBox(message, "gRabbit", wxOK | wxICON_ERROR, this);  
      return;
    }
    file.get( tmp, 6 );
    if ( std::strcmp( tmp, "Graph" ) != 0) {
      message.Printf("Invalid object type identifier!");
      wxMessageBox(message, "gRabbit", wxOK | wxICON_ERROR, this);  
      return;
    }
    file >> c >> arcs >> c >> vertex >> c >> graph >> c;
    if (!file.good()) {
      message.Printf("Invalid header file format: invalid parameters for Graph!");
      wxMessageBox(message, "gRabbit", wxOK | wxICON_ERROR, this);  
      return;
    }
    if (file.get() == '\n')
      line++;
  }
  
  unsigned long v1n,v2n,number,tmpn,x,y,r,g,b,state=0;
  wxString gg(""),strtmp("");
  GrabbitBasicShape *theShape;
  // Load Vertex list
  for ( unsigned long i=0; i < vertex; i++ ) {
    file >> c >> number;
    if ((c != 'V') || !file.good()) {
      message.Printf("Invalid vertex information on line %ld: Vertex Label", line);
      wxMessageBox(message, "gRabbit", wxOK | wxICON_ERROR, this);  
      return;
    }
    strtmp.Printf("V%ld",number);
    file >> c >> x >> c >> y >> c;
    if ( !file.good() ) {
      message.Printf("Invalid vertex information on line %ld: Vertex coordinates", line);
      wxMessageBox(message, "gRabbit", wxOK | wxICON_ERROR, this);  
      return;
    }
    state = 0;
    while ( file.get( c ) ) {
      switch (state)
	{
	case 0:
	  if (c == '"') state=1;
	  break;
	case 1:
	  if (c == '"') {
	    state=2;
	    break;
	  }
	  gg << c;
	  break;
	default:
	  message.Printf("Invalid internal state!");
	  wxMessageBox(message, "gRabbit", wxOK | wxICON_ERROR, this);  
	  return;
	  break;
	}
      if ((state == 2) || (c == EOF)) break;
    }
    if (state != 2) {
      message.Printf("Invalid vertex information on line %ld: Vertex Tip", line);
      wxMessageBox(message, "gRabbit", wxOK | wxICON_ERROR, this);
      return;
    }
    file >> c >> c >> r >> c >> g >> c >> b >> c;
    if ( !file.good() ) {
      message.Printf("Invalid vertex information on line %ld: Invalid Color", line);
      wxMessageBox(message, "gRabbit", wxOK | wxICON_ERROR, this);  
      return;
    }
    
    theShape = new GrabbitShapeVertex(wxPoint(x, y), wxSize(NODE_DEFAULT_SIZE, NODE_DEFAULT_SIZE), wxColour(r, g, b), gg);
    theShape->SetName( strtmp );
    all_shapes.push_back( theShape );
    
    gg.Clear();
    if (file.get() == '\n')
      line++;
  }
  // Load Arc list
  GrabbitShapeLine *theLine;
  GrabbitShapeVertex *v1,*v2;
  for ( long i=0; i < arcs; i++ ) {
    file >> c >> number;
    if ((c != 'A')) {
      message.Printf("Invalid arc information on line %ld: Arc Label", line);
      wxMessageBox(message, "gRabbit", wxOK | wxICON_ERROR, this);  
      return;
    }
    strtmp.Printf("A%ld",number);
    file >> c;
    if ( !file.good() ) {
      message.Printf("Invalid arc information on line %ld: Arc Label", line);
      wxMessageBox(message, "gRabbit", wxOK | wxICON_ERROR, this);  
      return;
    }
    state = 0;
    while ( file.get( c ) ) {
      switch (state)
	{
	case 0:
	  if (c == '"') state=1;
	  break;
	case 1:
	  if (c == '"') {
	    state=2;
	    break;
	  }
	  gg << c;
	  break;
	default:
	  message.Printf("Invalid internal state!");
	  wxMessageBox(message, "gRabbit", wxOK | wxICON_ERROR, this);  
	  return;
	  break;
	}
      if ((state == 2) || (c == EOF)) break;
    }
    if (state != 2) {
      message.Printf("Invalid arc information on line %ld: Arc Tip", line);
      wxMessageBox(message, "gRabbit", wxOK | wxICON_ERROR, this);
      return;
    }
    file >> c >> c >> r >> c >> g >> c >> b >> c >> c;
    if (!file.good()) {
      message.Printf("Invalid arc information on line %ld: Invalid Color", line);
      wxMessageBox(message, "gRabbit", wxOK | wxICON_ERROR, this);
      return;
    }
    file >> c >> v1n;
    if ( (c != 'V') || !file.good() ) {
      message.Printf("Invalid arc information on line %ld: Invalid Starting Vertex", line);
      wxMessageBox(message, "gRabbit", wxOK | wxICON_ERROR, this);
      return;
    }
    file >> c >> c >> v2n;
    if ( (c != 'V') || !file.good() ) {
      message.Printf("Invalid arc information on line %ld: Invalid Ending Vertex", line);
      wxMessageBox(message, "gRabbit", wxOK | wxICON_ERROR, this);
      return;
    }

    theLine = new GrabbitShapeArc(gg);
    gg.Printf("V%ld",v1n);
    for (it=all_shapes.begin();it != all_shapes.end();++it) {
      if ((*it)->GetName() == gg) {
	v1 = (GrabbitShapeVertex *)*it;
	break;
      }
    }
    gg.Printf("V%ld",v2n);
    for (it=all_shapes.begin();it != all_shapes.end();++it) {
      if ((*it)->GetName() == gg) {
	v2 = (GrabbitShapeVertex *)*it;
	break;
      }
    }
    theLine->SetVertex1(v1);
    theLine->SetVertex2(v2);
    theLine->SetName(strtmp);
    ((GrabbitShapeVertex *)v1)->AddLine(theLine);
    ((GrabbitShapeVertex *)v2)->AddLine(theLine);
    all_lines.push_back(theLine);
    
    gg.Clear();
    if (file.get() == '\n')
      line++;
  }
  actualGraph = filename;
  this->Refresh(false);
  ((wxFrame *)myParent)->SetStatusText(_T("Graph Loaded!"));

}

//----------------------------------------------------------------------------------------------------------------------
// Save the graph on disc.
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::SaveGraph(const wxString &filename) {
  wxString message("");

  ((wxFrame *)myParent)->SetStatusText(_T("Saving File. Please wait...."));
  
  std::ofstream file(filename, std::ios::out);
  if ( !file ) {
    message.Printf("Coldn't open file %s for writing", filename.c_str());   //: %s", filename.c_str(), filename.c_str());
    wxMessageBox(message, "gRabbit", wxOK | wxICON_ERROR, this);  
    return;
  }
  
  file << "GRABBIT-Graph(" << (long)all_lines.size() << "," << (long)all_shapes.size() << ",1)" << std::endl;
  { // Save Vertex's
    wxString tmpstr("");
    std::list< GrabbitBasicShape * >::iterator it;  
    for (it=all_shapes.begin();it != all_shapes.end();++it) {
      file << ((*it)->GetName()).c_str() << ","
	   << ((*it)->GetPosition()).x << ","
	   << ((*it)->GetPosition()).y << ",\""
	   << (((GrabbitShapeVertex *)*it)->GetLabel()).c_str() << "\",("
	   << (long)((*it)->GetColour()).Red() << ","
	   << (long)((*it)->GetColour()).Green() << ","
	   << (long)((*it)->GetColour()).Blue() << ")" << std::endl;
    }
  }

  { // Save Arc's
    wxString tmpstr("");
    std::list< GrabbitBasicShape * >::iterator it;  
    for (it=all_lines.begin();it != all_lines.end();++it) {
      file << ((*it)->GetName()).c_str() << ",\""
	   << (((GrabbitShapeArc *)*it)->GetLabel()).c_str() << "\",("
	   << (long)((*it)->GetColour()).Red() << ","
	   << (long)((*it)->GetColour()).Green() << ","
	   << (long)((*it)->GetColour()).Blue() << "),"
	   << (((GrabbitShapeArc *)*it)->GetVertex1())->GetName() << ","
	   << (((GrabbitShapeArc *)*it)->GetVertex2())->GetName() << std::endl;
    }
  }
  ((wxFrame *)myParent)->SetStatusText(_T("File Saved Successfully!"));
  message.Printf("File Saved Successfully!");
  wxMessageBox(message, "gRabbit", wxOK | wxICON_INFORMATION, this);
}



//----------------------------------------------------------------------------------------------------------------------
// Response to EVT_LEFT_DOWN
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::OnEvtLeftDown(wxMouseEvent& event) {
  wxPoint w(event.m_x, event.m_y);
  GrabbitHolder *h = new GrabbitHolder(w);
  
  (this->*(events[selected]))(this, h);
  delete h;
  
  event.Skip();
}

//----------------------------------------------------------------------------------------------------------------------
// Response to EVT_LEFT_UP
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::OnEvtLeftUp(wxMouseEvent& event) {
  if (holder != NULL)
    holder->RefreshImage();
  if (state != 1) {
    this->SetCursor(*wxSTANDARD_CURSOR);  
    holder = NULL;
  } else {
    this->SetCursor(*blankCursor);
    
    // If whe droped the line above an vertex, we associate both....
    long x = event.m_x, y = event.m_y;
    wxPoint cn;
    GrabbitBasicShape *clickedObj = NULL;
    std::list< GrabbitBasicShape * >::iterator it;
    for (it=all_shapes.begin();it != all_shapes.end();++it) {
      cn = (*it)->GetCentre();
      if ( (x - cn.x)*(x - cn.x) + (y - cn.y)*(y - cn.y) <= NODE_DEFAULT_RADIUS * NODE_DEFAULT_RADIUS ) {
	clickedObj = *it;
	break;
      }
    }
    if ((clickedObj != NULL) && (theLine->GetVertex1() != clickedObj)) {
      theLine->SetVertex2(clickedObj);
      theLine->GetVertex1()->RefreshImage();
      theLine = NULL;
      holder = NULL;
      state = 0;
      this->SetCursor(*wxSTANDARD_CURSOR);
      this->Refresh(false);
    }
  }
  this->Refresh(false);
}

//----------------------------------------------------------------------------------------------------------------------
// Response to EVT_MOTION
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::OnEvtMouseMove(wxMouseEvent& event) {
  long x = event.m_x, y = event.m_y;
  if (x < 0) x = 0;
  if (x > (bmpW - NODE_DEFAULT_SIZE)) x = bmpW - NODE_DEFAULT_SIZE;
  if (y < 0) y = 0;
  if (y > (bmpH - NODE_DEFAULT_SIZE)) y = bmpH - NODE_DEFAULT_SIZE;
  
  // Verify if some object was clicked
  if ( (holder != NULL) && event.m_leftDown) {
    holder->Move( x, y );
  } else if (theLine != NULL) {
    theLine->Move( x, y );
  } else { // If none, just return
    event.Skip();
    return;
  }
  this->Refresh(false); // Repaint all shapes....
}


//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::BtnSelected(wxWindow *w, GrabbitHolder *h) {
  // Verify if some object was clicked
  long x = h->GetX(), y = h->GetY();
  wxPoint cn;
  GrabbitBasicShape *clickedObj = NULL;
  std::list< GrabbitBasicShape * >::iterator it;
  for (it=all_shapes.begin();it != all_shapes.end();++it) {
    cn = (*it)->GetCentre();
    if ( (x - cn.x)*(x - cn.x) + (y - cn.y)*(y - cn.y) <= NODE_DEFAULT_RADIUS * NODE_DEFAULT_RADIUS ) {
      clickedObj = *it;
      break;
    }
  }
  
  // If "yes", place that on a global variable
  if ( clickedObj != NULL ) {
    holder = clickedObj;
    holder->Highlight();
    this->Refresh(false);
    this->SetCursor(*blankCursor);
  }
}

//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::BtnNode(wxWindow *w, GrabbitHolder *h) {
  long x = h->GetX(), y = h->GetY();
  wxPoint cnNew;
  // Draw the shape
  x-=8; y-=8;
  if ( x <= 0 ) { x = 1; }
  if ( y <= 0 ) { y = 1; }
  GrabbitBasicShape *theShape = new GrabbitShapeVertex(wxPoint(x, y), wxSize(NODE_DEFAULT_SIZE, NODE_DEFAULT_SIZE), *wxRED);
  cnNew = theShape->GetCentre();
  
  { // Checking where we are drawing. If there's something bellow, we negate the request. ;-)
    std::list< GrabbitBasicShape * >::iterator it;
    wxPoint cn;
    for (it=all_shapes.begin();it != all_shapes.end();++it) {
      // FIXME: We need some colision detection here. Just verify if the clicked point is inside
      // doesn't suffice. We most know if objects overlap!
      cn = (*it)->GetCentre();
      // Thanks to Wendel Scardua (wendelscardua@gmail.com) for correcting me in this test
      if ( (cnNew.x - cn.x)*(cnNew.x - cn.x) + (cnNew.y - cn.y)*(cnNew.y - cn.y) <= (NODE_DEFAULT_RADIUS + NODE_DEFAULT_RADIUS)*
	   (NODE_DEFAULT_RADIUS + NODE_DEFAULT_RADIUS) ) {
	// FIXME: Change here the cursor to indicate prohibition.
	this->SetCursor(wxCursor(wxCURSOR_NO_ENTRY));
	// Maybe a sound in windoze?
	delete theShape;
	return;
      }
    }
  }
  
  all_shapes.push_back(theShape);
  {
    wxMemoryDC mdc;
    mdc.SelectObject( *(this->myBmp) );
    theShape->Draw(mdc);
    mdc.SelectObject( wxNullBitmap );
  }
  this->Refresh(false);
}

//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::BtnLine(wxWindow *w, GrabbitHolder *h) {
  // - Verificar se o usuario clicou em cima de um n
  long x = h->GetX(), y = h->GetY();
  wxPoint cn;
  GrabbitBasicShape *clickedObj = NULL;
  std::list< GrabbitBasicShape * >::iterator it;
  for (it=all_shapes.begin();it != all_shapes.end();++it) {
    cn = (*it)->GetCentre();
    if ( (x - cn.x)*(x - cn.x) + (y - cn.y)*(y - cn.y) <= NODE_DEFAULT_RADIUS*NODE_DEFAULT_RADIUS ) {
      clickedObj = *it;
      break;
    }
  }

  // - Se clicou, dar um highlight no n clicado e entrar para o estado de desenho de reta.
  if (clickedObj != NULL) {
    switch (state)
      {
      case 0:
	theLine = new GrabbitShapeArc("");
	theLine->SetVertex1(clickedObj);
	clickedObj->Highlight();
	theLine->Move(x,y);
	all_lines.push_back(theLine);
	((wxFrame *)myParent)->SetStatusText(_T(""));
	this->SetCursor(*blankCursor);
	state = 1;
	break;
      case 1:
	theLine->GetVertex1()->RefreshImage();
	((wxFrame *)myParent)->SetStatusText(_T(""));
	state = 0;
	if (clickedObj == theLine->GetVertex1()) { // Same vertex. Cancel operation.
	  std::list< GrabbitBasicShape * >::iterator el = std::find(all_lines.begin(), all_lines.end(), theLine);
	  if (el != all_lines.end()) {
	    el = all_lines.erase(el);
	  }
	  theLine->SetVertex1(NULL);
	  delete theLine;
	  theLine = NULL;
	  break;
	}
	theLine->SetVertex2(clickedObj);
	theLine = NULL;
	break;
      default:
	state = 0;
	break;
      }
    this->Refresh(false);
  } else { // - Se no clicou, exibe o cursor de negao e retorna
    this->SetCursor(wxCursor(wxCURSOR_NO_ENTRY));
    if (state == 0) {
      ((wxFrame *)myParent)->SetStatusText(_T("Click inside a vertex please!"));
    } else {
      ((wxFrame *)myParent)->SetStatusText(_T("Click inside another vertex please!"));
    }
    return;
  }
  

  // - Se ele j est no estado de desenho de reta, terminar de desenhar a reta e associar
  //   ela aos ns existentes nas pontas. Desselecionar os objetos.

  
}

//----------------------------------------------------------------------------------------------------------------------
// OnPaint: OnPaint response
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::OnPaint(wxPaintEvent& event) {
  wxPaintDC dc(this);
  { // Repaint All Shapes! :-)
    wxMemoryDC mdc;
    mdc.SelectObject( *(this->myBmp) );
    mdc.SetBackground( wxBrush( (GrabbitMainFrame::GetConfigurator())->GetDrawAreaBgColor(), wxSOLID) );
    mdc.Clear();
    std::list< GrabbitBasicShape * >::iterator it;
    for (it=all_lines.begin();it != all_lines.end();++it) { // Lines first! ;-)
      (*it)->Draw(mdc);
    }
    for (it=all_shapes.begin();it != all_shapes.end();++it) {
      (*it)->Draw(mdc);
    }
    if(holder != NULL)
      holder->Draw(mdc);
    if (theLine != NULL)
      theLine->Draw(mdc);
    mdc.SelectObject( wxNullBitmap );
  }
  dc.DrawBitmap(*(this->myBmp), wxPoint(0,0), false);
}

//----------------------------------------------------------------------------------------------------------------------
// OnResize
//----------------------------------------------------------------------------------------------------------------------
// Thanks to Wagner Cesar Bruna (wbruna@yahoo.com) for some hints on this function.
void GrabbitMainScrolledWindow::OnResize(wxSizeEvent &event) {
  int w, h;
  int newW, newH;
  int dw, dh;
  { // To get the scrollbar's sizes.
    wxSize mySize(this->GetSize());
    wxSize clSize(this->GetClientSize());
    dw = mySize.GetWidth() - clSize.GetWidth();
    dh = mySize.GetHeight() - clSize.GetHeight();
  }
  myParent->GetClientSize(&w,&h);
  
  if (myBmp) delete myBmp;
  newW = (w>bmpW)?w-(26*4+20):bmpW;
  newH = (h>bmpH)?h:bmpH;
  myBmp = new wxBitmap(newW, newH, -1);
  {
    wxMemoryDC mdc;
    mdc.SelectObject( *(this->myBmp) );
    mdc.SetBackground( wxBrush( (GrabbitMainFrame::GetConfigurator())->GetDrawAreaBgColor(), wxSOLID) );
    mdc.Clear();
    mdc.SelectObject( wxNullBitmap );
  }
  this->SetClientSize(wxSize(w-(26*4+20)-dw, h-dh));
  this->SetScrollbars(1, 1, newW - dw +1, newH - dh +1);
  this->Refresh(false); // This take care to redraw the entire client area on OnPaint()
}

//----------------------------------------------------------------------------------------------------------------------
// Aux
//----------------------------------------------------------------------------------------------------------------------
void GrabbitMainScrolledWindow::ResizeBitmapArea(int w, int h) {
  bmpW = w;
  bmpH = h;
  if (myBmp) delete myBmp;
  myBmp = new wxBitmap(w-(26*4+20),h,-1);
  {
    wxMemoryDC mdc;
    mdc.SelectObject( *(this->myBmp) );
    mdc.SetBackground( wxBrush( (GrabbitMainFrame::GetConfigurator())->GetDrawAreaBgColor(), wxSOLID) );
    mdc.Clear();
    mdc.SelectObject( wxNullBitmap );
    wxPaintDC dc(this);
    dc.DrawBitmap(*(this->myBmp), wxPoint(0,0), FALSE);
  }
  this->SetScrollbars(1, 1, bmpW, bmpH);
  this->Refresh(false);  // This take care to redraw the entire client area on OnPaint()
}
