//----------------------------------------------------------------------------------------------------------------------
//
//  gRabbit - Graph Builder for Basic Implementation Tests
//  Copyright 2004, Nelson Guedes Paulo Junior (npaulo@linux.ime.usp.br)
//
//  Under gRabbit license (see LICENSE.TXT for details)
//
//
//  - GrabbitMainScrolledWindow: Part of main window (interface)
// 
//----------------------------------------------------------------------------------------------------------------------
#ifndef __GRABBIT_MAIN_SCROLLED_WINDOW_H_
#define __GRABBIT_MAIN_SCROLLED_WINDOW_H_

#ifdef __GNUG__
  #pragma interface
#endif

// gRabbit includes
#include "grabbit_common.h"
#include "grabbit_basic_shape.h"
#include "grabbit_holder.h"
#include "grabbit_shape_line.h"
#include "grabbit_animation_iterator.h"

// C++ Library includes
#include <list>
#include <vector>


//----------------------------------------------------------------------------------------------------------------------
// To use member functions
//----------------------------------------------------------------------------------------------------------------------
#define CALL_MEMBER_FN(object, ptrToMember) ((object).*(ptrToMember))

class GrabbitMainScrolledWindow;
typedef void (GrabbitMainScrolledWindow::*EvtFunction)(wxWindow *, GrabbitHolder *);
typedef void (GrabbitMainScrolledWindow::*AniEvtFunction)();


//----------------------------------------------------------------------------------------------------------------------
// GrabbitScrolledWindow Class
//----------------------------------------------------------------------------------------------------------------------
class GrabbitMainScrolledWindow : public wxScrolledWindow
{
 public:
  // Constructors and Destructors
  GrabbitMainScrolledWindow(wxWindow *parent, const wxPoint& pos, const wxSize& size );
  ~GrabbitMainScrolledWindow();

  // Other Methods
  void ResizeBitmapArea(int w, int h);
  void LoadGraph(const wxString &filename);
  void SaveGraph(const wxString &filename);
  
  // Animation
  void LoadAnimation(const wxString &filename);
  void StartTimer();
  void StopTimer();
  void SetBtnPlay(bool b) { btnPlay = b; }
  void ExecAnimationHandlerFunction(unsigned int i);
  
  // Handlers
  void OnResize(wxSizeEvent &event);
  void OnPaint(wxPaintEvent& event);
  void OnEvtLeftDown(wxMouseEvent& event);
  void OnEvtLeftUp(wxMouseEvent& event);
  void OnEvtMouseMove(wxMouseEvent& event);
  void OnTimer(wxTimerEvent& event);
  
  // Events...
  void BtnSelected(wxWindow *w, GrabbitHolder *h);
  void BtnNode(wxWindow *w, GrabbitHolder *h);
  void BtnLine(wxWindow *w, GrabbitHolder *h);

  // Animation buttons
  void AniButtonPlay();
  void AniButtonStop();
  void AniButtonPause();
  void AniButtonRound();
  void AniButtonStepBack();
  void AniButtonStepNext();
  void AniButtonRewind();
  void AniButtonForward();

  
  // Static
  static void SetEventHandlerFunction(unsigned int i) { selected = i; }
  static unsigned int GetEventHandlerFunction() { return selected; }
  static unsigned int GetInternalState() { return state; }
  static void SetDelay(unsigned long d) { delay = d; }
  static GrabbitMainScrolledWindow *GetMe() { return instance; }
  
 private:
  // Methods
  bool ProcessNextInstruction(GrabbitInstructions *undo = NULL);

  // Variables
  wxWindow *myParent;
  wxBitmap *myBmp;
  wxBitmap *myBuffer;
  wxCursor *blankCursor;
  
  GrabbitBasicShape *holder;
  GrabbitShapeLine *theLine;

  long bmpW, bmpH;

  static GrabbitMainScrolledWindow *instance;
  static unsigned int selected;
  static unsigned int state;
  static unsigned long delay;
  static wxMutex *aniMutex;
  std::vector< EvtFunction > events;
  std::vector< AniEvtFunction > ani_events;

  // Graph variables
  wxString actualGraph;
  std::list< GrabbitBasicShape * > all_shapes;
  std::list< GrabbitBasicShape * > all_lines;
  bool directed;

  // Animation variables
  wxTimer myTimer;
  bool btnPlay;
  bool btnPause;
  bool btnRound;
  bool aniRestarts;
  GrabbitAnimationIterator *myAniIt;
  
  // Event Table
  DECLARE_EVENT_TABLE();
};


#endif
