//----------------------------------------------------------------------------------------------------------------------  
//
//  gRabbit - Graph Builder for Basic Implementation Tests
//  Copyright 2004, Nelson Guedes Paulo Junior (npaulo@linux.ime.usp.br)
//
//  Under gRabbit license (see LICENSE.TXT for details)
//
//
//  - GrabbitShapeArc: An Arc Class for gRabbit (implementation)
// 
//----------------------------------------------------------------------------------------------------------------------  
#ifdef __GNUG__
  #pragma implementation
#endif

// gRabbit includes
#include "grabbit_common.h"
#include "grabbit_shape_arc.h"
#include "grabbit_basic_shape.h"
#include "grabbit_shape_line.h"
#include "grabbit_shape_vertex.h"

// C++ includes
#include <cmath>
#include <iostream>

//----------------------------------------------------------------------------------------------------------------------
// Number of created objects
//----------------------------------------------------------------------------------------------------------------------
unsigned long GrabbitShapeArc::created=0;

//----------------------------------------------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------------------------------------------
GrabbitShapeArc::GrabbitShapeArc(const wxString& ll)
  : GrabbitShapeLine()
{
  label = ll;
  {
    created++;
    wxString tmpstr("");
    tmpstr.Printf("A%lu", created);
    name = tmpstr;
  }
}

//----------------------------------------------------------------------------------------------------------------------
// Draw the line
//----------------------------------------------------------------------------------------------------------------------
void GrabbitShapeArc::Draw(wxDC& dc)
{
#include "sin_cos_table.h"
  double xa, ya, xb, yb, xc, yc;
  double xd, yd, xe, ye, xf, yf;
  double c, s;
  
  long p2x, p2y;
  if ( v2 != NULL ) {
    p2x = (v2->GetCentre()).x;
    p2y = (v2->GetCentre()).y;
    xa = -9 -NODE_DEFAULT_RADIUS; ya = -3;
    xb = -9 -NODE_DEFAULT_RADIUS; yb =  3;
    xc = -NODE_DEFAULT_RADIUS;  yc = 0;
  } else {
    p2x = shape_pos.x;
    p2y = shape_pos.y;
    xa = -9; ya = -3;
    xb = -9; yb =  3;
    xc = -0;  yc = 0;
  }

  int index = RAD(std::atan2((double)(p2y - (v1->GetCentre()).y), (double)(p2x - (v1->GetCentre()).x)));
  if (index < 0)  index = index + 360;
  
  c = cosine[index];
  s = sine[index];
  
  xd =  c * xa + s * ya;
  yd = -s * xa + c * ya;
  
  xe =  c * xb + s * yb;
  ye = -s * xb + c * yb;
  
  xf =  c * xc + s * yc;
  yf = -s * xc + c * yc;

  long x1 = (long)(xf + p2x);
  long y1 = (long)(p2y - yf);
  
  long x2 = (long)(xd + p2x);
  long y2 = (long)(p2y - yd);
  
  long x3 = (long)(xe + p2x);
  long y3 = (long)(p2y - ye);

  wxPoint points[3];
  points[0] = wxPoint(x1, y1);
  points[1] = wxPoint(x2, y2);
  points[2] = wxPoint(x3, y3);


  dc.SetPen( wxPen( this->shape_col, this->shape_pen_size, wxSOLID) );
  dc.SetBrush( wxBrush( this->shape_col, wxSOLID ) );
  
  dc.DrawPolygon(3, points);
  dc.DrawLine((v1->GetCentre()).x, (v1->GetCentre()).y, x1, y1);

  
  // Text
  long w, h;
  dc.SetFont( wxFont(14, wxMODERN, wxNORMAL, wxBOLD, false) );
  dc.GetTextExtent(label, &w, &h);
  dc.SetTextBackground( this->shape_col );
  dc.SetTextForeground( *(wxTheColourDatabase->FindColour("BLUE")) );
  
  xa = -(w/2); ya = h;
  xb = w/2;    yb = 0;

  xc =  c * xa + s * ya;
  yc = -s * xa + c * ya;

  xd =  c * xb + s * yb;
  yd = -s * xb + c * yb;

  long mx = ((v1->GetCentre()).x + p2x) /2;
  long my = ((v1->GetCentre()).y + p2y) /2;
  
  x1 = (long)(xc + mx);
  y1 = (long)(my - yc);
  
  x2 = (long)(xd + mx);
  y2 = (long)(my - yd);

  if (index > 90 && index <= 270 ) {
    x1 = x2;
    y1 = y2;
    index+=180;
  }
  dc.DrawRotatedText(label, x1, y1, (double)- index );
  
}

//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
void GrabbitShapeArc::Highlight()
{

}
