//----------------------------------------------------------------------------------------------------------------------  
//
//  gRabbit - Graph Builder for Basic Implementation Tests
//  Copyright 2004, Nelson Guedes Paulo Junior (npaulo@linux.ime.usp.br)
//
//  Under gRabbit license (see LICENSE.TXT for details)
//
//
// - Vertex Shape (implementation)
// 
//----------------------------------------------------------------------------------------------------------------------  
#ifdef __GNUG__
  #pragma implementation
#endif

// gRabbit includes
#include "grabbit_common.h"
#include "grabbit_basic_shape.h"
#include "grabbit_shape_vertex.h"
#include "grabbit_shape_line.h"

// C++ includes
#include <cstdlib>
#include <iostream>

//----------------------------------------------------------------------------------------------------------------------
// Number of created objects
//----------------------------------------------------------------------------------------------------------------------
unsigned long GrabbitShapeVertex::created=0;

//----------------------------------------------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------------------------------------------
GrabbitShapeVertex::GrabbitShapeVertex(const wxPoint &pos, const wxSize &tam, const wxColour &col)
  : GrabbitBasicShape(pos, tam, col)
{
  {
    created++;
    wxString tmpstr("");
    tmpstr.Printf("V%ld",created);
    name = tmpstr;
  }
  { // Create the drawing
    wxMemoryDC mdc;
    mdc.SelectObject( this->shape_bmp );
    mdc.SetBackground( wxBrush(*wxWHITE, wxTRANSPARENT) );
    mdc.Clear();
    mdc.SetPen( wxPen(*wxBLACK, 1, wxSOLID) );
    mdc.DrawEllipse( wxPoint(0,0), wxSize(shape_tam.x-1, shape_tam.y-1) );
    mdc.SetBrush( wxBrush( this->shape_col, wxSOLID) );
    mdc.FloodFill( wxPoint((this->shape_tam).x/2,(this->shape_tam).y/2), *wxBLACK, wxFLOOD_BORDER );
    if (label.Length()) {
      long w, h;
      mdc.SetFont( wxFont(10,wxMODERN,wxNORMAL,wxBOLD,false) );
      mdc.GetTextExtent(label, &w, &h);
      mdc.SetTextBackground( this->shape_col );
      mdc.SetTextForeground( *(wxTheColourDatabase->FindColour("YELLOW")) );
      mdc.DrawText(label, (17 - w)/2 , (17 - h)/2 );
    }
    mdc.SelectObject( wxNullBitmap );
  }
  wxBitmap bitmask(tam.x, tam.y, -1);
  {
    wxMemoryDC mdc;
    mdc.SelectObject( bitmask );
    mdc.SetBackground( wxBrush(*wxWHITE, wxTRANSPARENT) );
    mdc.Clear();
    mdc.SetPen( wxPen(*wxBLACK, 1, wxSOLID) );
    mdc.DrawEllipse( wxPoint(0,0), wxSize(shape_tam.x-1, shape_tam.y-1) );
    mdc.SetBrush( wxBrush( *wxBLACK, wxSOLID) );
    mdc.FloodFill( wxPoint((this->shape_tam).x/2,(this->shape_tam).y/2), *wxBLACK, wxFLOOD_BORDER );
    mdc.SelectObject( wxNullBitmap );
  }
  (this->shape_bmp).SetMask(new wxMask(bitmask, *wxWHITE));
  
}

//----------------------------------------------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------------------------------------------
GrabbitShapeVertex::GrabbitShapeVertex(const wxPoint &pos, const wxSize &tam, const wxColour &col, const wxString &ll)
  : GrabbitBasicShape(pos, tam, col)
{
  label = ll;
  {
    created++;
    wxString tmpstr("");
    tmpstr.Printf("V%ld",created);
    name = tmpstr;
  }
  { // Create the drawing
    wxMemoryDC mdc;
    mdc.SelectObject( this->shape_bmp );
    mdc.SetBackground( wxBrush(*wxWHITE, wxTRANSPARENT) );
    mdc.Clear();
    mdc.SetPen( wxPen(*wxBLACK, 1, wxSOLID) );
    mdc.DrawEllipse( wxPoint(0,0), wxSize(shape_tam.x-1, shape_tam.y-1) );
    mdc.SetBrush( wxBrush( this->shape_col, wxSOLID) );
    mdc.FloodFill( wxPoint((this->shape_tam).x/2,(this->shape_tam).y/2), *wxBLACK, wxFLOOD_BORDER );
    if (label.Length()) {
      long w, h;
      mdc.SetFont( wxFont(12, wxMODERN, wxNORMAL, wxBOLD, false) );
      mdc.GetTextExtent(label, &w, &h);
      mdc.SetTextBackground( this->shape_col );
      mdc.SetTextForeground( *(wxTheColourDatabase->FindColour("YELLOW")) );
      mdc.DrawText(label, (NODE_DEFAULT_SIZE - w - 1)/2 , (NODE_DEFAULT_SIZE - h)/2 );
    }
    mdc.SelectObject( wxNullBitmap );
  }
  wxBitmap bitmask(tam.x, tam.y, -1);
  {
    wxMemoryDC mdc;
    mdc.SelectObject( bitmask );
    mdc.SetBackground( wxBrush(*wxWHITE, wxTRANSPARENT) );
    mdc.Clear();
    mdc.SetPen( wxPen(*wxBLACK, 1, wxSOLID) );
    mdc.DrawEllipse( wxPoint(0,0), wxSize(shape_tam.x-1, shape_tam.y-1) );
    mdc.SetBrush( wxBrush( *wxBLACK, wxSOLID) );
    mdc.FloodFill( wxPoint((this->shape_tam).x/2,(this->shape_tam).y/2), *wxBLACK, wxFLOOD_BORDER );
    mdc.SelectObject( wxNullBitmap );
  }
  (this->shape_bmp).SetMask(new wxMask(bitmask, *wxWHITE));
  
}


//----------------------------------------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------------------------------------
GrabbitShapeVertex::~GrabbitShapeVertex() {
}

//----------------------------------------------------------------------------------------------------------------------
// Redraw the bitmap deep inside of this object. :-)
//----------------------------------------------------------------------------------------------------------------------
void GrabbitShapeVertex::RefreshImage() {
    wxMemoryDC mdc;
    mdc.SelectObject( this->shape_bmp );
    mdc.SetBackground( wxBrush(bg_colour, wxTRANSPARENT) );
    mdc.Clear();
    mdc.SetPen( wxPen(*wxBLACK, 1, wxSOLID) );
    mdc.DrawEllipse( wxPoint(0,0), wxSize(shape_tam.x-1, shape_tam.y-1) );
    mdc.SetBrush( wxBrush( this->shape_col, wxSOLID) );
    mdc.FloodFill( wxPoint((this->shape_tam).x/2,(this->shape_tam).y/2), *wxBLACK, wxFLOOD_BORDER );
    if (label.Length()) {
      long w, h;
      mdc.SetFont( wxFont(12, wxMODERN, wxNORMAL, wxBOLD, false) );
      mdc.GetTextExtent(label, &w, &h);
      mdc.SetTextBackground( this->shape_col );
      mdc.SetTextForeground( *(wxTheColourDatabase->FindColour("YELLOW")) );
      mdc.DrawText(label, (NODE_DEFAULT_SIZE - w - 1)/2 , (NODE_DEFAULT_SIZE - h)/2 );
    }
    mdc.SelectObject( wxNullBitmap );
}

//----------------------------------------------------------------------------------------------------------------------
// Answers true if the given coordinates are inside the object, and false otherwise.
//----------------------------------------------------------------------------------------------------------------------
// FIXME: This is wrong! It's true for a square.
bool GrabbitShapeVertex::GotInside(long x, long y) const {
  if ( (labs(x - GetCentre().x) <= (this->shape_tam).x) &&
       (labs(y - GetCentre().y) <= (this->shape_tam).y)) {
    return true;
  }
  return false;
}


//----------------------------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
void GrabbitShapeVertex::Highlight() {
  wxMemoryDC mdc;
  mdc.SelectObject( this->shape_bmp );
  mdc.SetBackground( wxBrush(bg_colour, wxTRANSPARENT) );
  mdc.Clear();
  wxColour *tmpcolour = wxTheColourDatabase->FindColour("BLACK");
  wxColour *tmpcolour2 = wxTheColourDatabase->FindColour("SPRING GREEN");
  mdc.SetPen( wxPen( *tmpcolour, 1, wxSOLID) );
  mdc.DrawEllipse( wxPoint(0,0), wxSize(shape_tam.x-1, shape_tam.y-1) );
  mdc.SetBrush( wxBrush( *tmpcolour2 , wxSOLID) );
  mdc.FloodFill( wxPoint((this->shape_tam).x/2,(this->shape_tam).y/2), *tmpcolour, wxFLOOD_BORDER );
  mdc.SelectObject( wxNullBitmap );
}


//----------------------------------------------------------------------------------------------------------------------
// Add a new line to the adjacency list.
//----------------------------------------------------------------------------------------------------------------------
void GrabbitShapeVertex::AddLine(GrabbitShapeLine *line) {
  adjacency.push_back( line );
}

//----------------------------------------------------------------------------------------------------------------------
// Remove a line from the adjacency list.
//----------------------------------------------------------------------------------------------------------------------
bool GrabbitShapeVertex::RemoveLine (GrabbitShapeLine *line) {
  std::list< GrabbitShapeLine * >::iterator it;
  for (it=adjacency.begin();it != adjacency.end();++it) {
    if (*it == line) {
      it = adjacency.erase(it);
      return true;
    }
  }
  return false;
}

//----------------------------------------------------------------------------------------------------------------------
// Tests if a line belongs to the adjacency list.
//----------------------------------------------------------------------------------------------------------------------
bool GrabbitShapeVertex::BelongsTo(GrabbitShapeLine *line) {
  std::list< GrabbitShapeLine * >::iterator el = std::find(adjacency.begin(), adjacency.end(), line);
  if (el != adjacency.end()) {
    return true;
  }
  return false;
}
